/*
Copyright (C) Christopher Fogelberg, Mengjie Zhang 2005

This file is part of VUWLGP.

VUWLGP is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

VUWLGP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with VUWLGP; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
  
#ifndef INSTRUCTIONOPERATIONPLUS_H
#define INSTRUCTIONOPERATIONPLUS_H

#include "Config.h"
#include "IInstructionArgument.h"
#include "InstructionArgumentRegister.h"
#include "IInstructionOperation.h"
#include "RegisterCollection.h"

namespace VUWLGP {
  // Represents an addition operation
  template<class T> class InstructionOperationPlus: public IInstructionOperation<T> {
  public:
    virtual ~InstructionOperationPlus() { }

    bool Execute(InstructionArgumentRegister<T>* dest, 
		 const IInstructionArgument<T>* const first,
		 const IInstructionArgument<T>* const second,
		 FitnessEnvironment<T>& fe) const;

    IInstructionOperation<T>* Clone() const;

    std::string ToString() const { return "+"; }
  
    static IInstructionOperation<T>* Generate(Config<T>* conf);
  };
}



///////////////////////////////// INLINE FUNCTION DEFINITIONS //////////////////////////////// 
template <class T> class VUWLGP::IInstructionOperation;

// Adds first to second and writes the result to dest. Returns true to indicate that
// execution should proceed to the next instruction.
template <class T> bool
VUWLGP::InstructionOperationPlus<T>::Execute(InstructionArgumentRegister<T>* dest,
						  const IInstructionArgument<T>* const first,
						  const IInstructionArgument<T>* const second,
						  FitnessEnvironment<T>& fe) const {
  fe.WriteRegister(dest->Index(), first->Value(fe) + second->Value(fe));
  return true;
}



template <class T> VUWLGP::IInstructionOperation<T>* 
VUWLGP::InstructionOperationPlus<T>::Clone() const {
  return static_cast<IInstructionOperation<T>*>(new InstructionOperationPlus());
}



template <class T> VUWLGP::IInstructionOperation<T>* 
VUWLGP::InstructionOperationPlus<T>::Generate(VUWLGP::Config<T>* conf) {
  return new InstructionOperationPlus();
}
#endif
