/*
Copyright (C) Christopher Fogelberg, Mengjie Zhang 2005

This file is part of VUWLGP.

VUWLGP is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

VUWLGP is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with VUWLGP; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
  
#include <iostream>
#include <iomanip>

#include "MultiClassProgram.h"
#include "Config.h"
#include "Instruction.h"

#include "IInstructionArgument.h"
#include "InstructionArgumentConstant.h"
#include "InstructionArgumentFeature.h"
#include "InstructionArgumentRegister.h"

#include "IInstructionOperation.h"
#include "IInstructionOperationConditional.h"
#include "InstructionOperationIflt.h"
#include "InstructionOperationPlus.h"

#include "MultiClassFitnessCase.h"

int main(int argc, char** argv) {
  // Configure the Config object:
  VUWLGP::Config<double>* config = new VUWLGP::Config<double>();
  config->numFeatures = 3;
  config->numRegisters = 10;

  config->argumentGenerators->AddElement(VUWLGP::InstructionArgumentConstant<double>::Generate);
  config->argumentGenerators->AddElement(VUWLGP::InstructionArgumentFeature<double>::Generate);
  config->argumentGenerators->AddElement(VUWLGP::InstructionArgumentRegister<double>::Generate);

  config->instructionOperations->AddElement(VUWLGP::InstructionOperationPlus<double>::Generate);
  config->instructionOperations->AddElement(VUWLGP::InstructionOperationIflt<double>::Generate);

  // Create random instructions, display them and execute them in some fitness environment.
  VUWLGP::FitnessEnvironment<double> fe(config);
  fe.WriteRegister(0, 0);
  fe.WriteRegister(1, 0);
  fe.WriteRegister(2, 0);
  fe.WriteRegister(3, 0);
  fe.WriteRegister(4, 0);
  fe.WriteRegister(5, 0);
  fe.WriteRegister(6, 0);
  fe.WriteRegister(7, 0);
  fe.WriteRegister(8, 0);
  fe.WriteRegister(9, 0);
  fe.AddCase(new VUWLGP::MultiClassFitnessCase<double>("fileName x y 1 className 1.5 -3 0.2"));

  std::cout << "Testing IProgram and BasicProgram" << std::endl;
  std::cout << "(1) Creating a random program and executing it. Conditionals are respected."
	    << std::endl;

  VUWLGP::IProgram<double>* prog = new VUWLGP::MultiClassProgram<double>(12, config);
  prog->MarkIntrons();
  std::cout << "Program:\n" << prog->ToString(true, true) << std::endl;
  
  std::cout << "\nStarting FitnessEnvironment:" << std::endl;
  std::cout << fe.ToString() << std::endl;

  prog->Execute(fe);

  std::cout << "\nResulting FitnessEnvironment:" << std::endl;
  std::cout << fe.ToString() << std::endl;

  std::cout << "\n(2) Result of culling program to size 10." << std::endl;
  prog->RandomlyCullToSize(10);
  prog->MarkIntrons();
  std::cout << "Program:\n" << prog->ToString(true, true) << std::endl;

  delete prog;
  delete config;

  return 0;
}
